<?php
/*
Plugin Name: Woocommerce Medianet
Plugin URI: http://www.medianetpay.ec
Description: Modulo de MedianetPay para WooCommerce
Version: 1.36
Author: EnLineaPagos - WPOSS
Author URI: http://www.medianetpay.ec
*/

if (!isset($start)) :
    add_action('plugins_loaded', 'woocommerce_medianetpay_init', 0);
    add_action('plugins_loaded', 'medianetpayshortcuts', 0);
endif;

function medianetpayshortcuts()
{
    add_shortcode('medianetpayresponseview', 'medianetpayresponseview');
}

function medianetpayresponseview()
{
    WC_MedianetPay::showConfirmation();
}

function woocommerce_medianetpay_init()
{
    if (!class_exists('WC_Payment_Gateway')) return;

    class WC_MedianetPay extends WC_Payment_Gateway
    {
        public function __construct()
        {
            /* ------------ Change data and logo for others clients -------------- */
            $this->method_title = 'MedianetPay';
            $this->key_licenses = 'd55404bd3dd8_MedianetPay';
            $this->url_develop = 'https://qa-plataforma.medianetpay.ec/secure/gangway/index.do';
            $this->url_production = 'https://plataforma.medianetpay.ec/secure/gangway/index.do';
            /* ------------------------------------------------------------------- */
            $this->id = 'medianetpay';
            $this->medthod_title = 'MedianetPay';
            $this->icon = apply_filters('woocommerce_payment_icon', plugins_url('/img/logo.png', __FILE__));
            $this->has_fields = false;

            $this->init_form_fields();
            $this->init_settings();

            $this->title = $this->settings['title'];
            $this->description = $this->settings['description'];
            $this->merchant_id = $this->settings['merchant_id'];
            $this->salt = $this->settings['salt'];
            $this->redirect_page_id = $this->settings['redirect_page_id'];
            $this->liveurl = ($this->settings['gateway_url'] == 0) ? $this->url_develop : $this->url_production;

            $this->msg['message'] = "";
            $this->msg['class'] = "";

            add_action('init', array(&$this, 'check_medianetpay_response'));
            if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
            } else {
                add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
            }
            add_action('woocommerce_receipt_medianetpay', array(&$this, 'receipt_page'));
        }

        function init_form_fields()
        {

            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Activar/Desactivar', 'medianetpay'),
                    'type' => 'checkbox',
                    'label' => __('Activar el Modulo de Medianet.', 'medianetpay'),
                    'default' => 'no'
                ),
                'title' => array(
                    'title' => __('Titulo:', 'medianetpay'),
                    'type' => 'text',
                    'description' => __('Esto controla el titulo durante el proceso de compra.', 'medianetpay'),
                    'default' => __('MedianetPay', 'medianetpay')
                ),
                'description' => array(
                    'title' => __('Descripcion:', 'medianetpay'),
                    'type' => 'textarea',
                    'description' => __('Esto controla la descripción durante el proceso de pago del cliente.', 'medianetpay'),
                    'default' => __('Pague facilmente desde MedianetPay con tarjetas crédito y débito.', 'medianetpay')
                ),
                'merchant_id' => array(
                    'title' => __('Usuario', 'medianetpay'),
                    'type' => 'text',
                    'description' => __('Proporcionado en el correo de Bienvenida de MedianetPay.')
                ),
                'salt' => array(
                    'title' => __('Llave Md5', 'medianetpay'),
                    'type' => 'text',
                    'description' => __('Proporcionado  en el correo de Bienvenida de MedianetPay', 'medianetpay'),
                ),
                'gateway_url' => array(
                    'title' => __('Ambiente', 'payment_gateway'),
                    'type' => 'select',
                    'options' => ['Desarrollo', 'Producción'],
                    'description' => __('Ambiente para la pasarela de pago ' . $this->method_title, 'payment_gateway')
                ),
                'redirect_page_id' => array(
                    'title' => __('Página de finalizacion del pago'),
                    'type' => 'select',
                    'options' => $this->get_pages('Seleccione una página'),
                    'description' => "URL de pagina de pago exitosa"
                )
            );
        }

        public function admin_options()
        {
            echo '<h3>' . __('MedianetPay', 'medianetpay') . '</h3>';
            echo '<p>' . __('MedianetPay es la Plataforma Nro 1 de Pagos Electrónicos en Ecuador') . '</p>';
            echo '<table class="form-table">';

            $this->generate_settings_html();
            echo '</table>';
        }


        function payment_fields()
        {
            if ($this->description) echo wpautop(wptexturize($this->description));
        }

        function receipt_page($order)
        {
            echo '<p>' . __('Gracias por su orden, por favor haga click en el botón de abajo para ir a pagar desde MedianetPay.', 'medianetpay') . '</p>';
            echo $this->generate_medianetpay_form($order);
        }

        public function generate_medianetpay_form($order_id)
        {

            global $woocommerce;
            $order = new WC_Order($order_id);
            $txnid = $order_id . '_' . date("ymds");

            $redirect_url = ($this->redirect_page_id == "" || $this->redirect_page_id == 0) ? get_site_url() . "/" : get_permalink($this->redirect_page_id);

            $productinfo = "Order $order_id";

            $str = "$this->merchant_id|$txnid|$order->order_total|$productinfo|$order->billing_first_name|$order->billing_email|||||||||||$this->salt";
            $hash = hash('sha512', $str);

            $medianetpay_args = array(
                'key' => $this->merchant_id,
                'txnid' => $txnid,
                'amount' => $order->order_total,
                'productinfo' => $productinfo,
                'firstname' => $order->billing_first_name,
                'lastname' => $order->billing_last_name,
                'address1' => $order->billing_address_1,
                'address2' => $order->billing_address_2,
                'city' => $order->billing_city,
                'state' => $order->billing_state,
                'country' => $order->billing_country,
                'zipcode' => $order->billing_zip,
                'email' => $order->billing_email,
                'phone' => $order->billing_phone,
                'surl' => $redirect_url,
                'furl' => $redirect_url,
                'curl' => $redirect_url,
                'hash' => $hash,
                'pg' => 'NB'
            );

            $medianetpay_args_array = array();
            foreach ($medianetpay_args as $key => $value) {
                $medianetpay_args_array[] = "<input type='hidden' name='$key' value='$value'/>";
            }
            return '<form action="' . $this->liveurl . '" method="post" id="medianetpay_payment_form">
            ' . implode('', $medianetpay_args_array) . '
            <input type="submit" class="button-alt" id="submit_medianetpay_payment_form" value="' . __('Pagar desde MedianetPay', 'medianetpay') . '" /> <a class="button cancel" href="' . $order->get_cancel_order_url() . '">' . __('Cancel order &amp; restore cart', 'medianetpay') . '</a>
            <script type="text/javascript">
jQuery(function(){
jQuery("body").block(
        {
            message: "<img src=\"' . $woocommerce->plugin_url() . '/assets/images/ajax-loader.gif\" alt=\"Redirecting…\" style=\"float:left; margin-right: 10px;\" />' . __('Gracias por su Orden, Estamos reenviandolo a la Pasarela de Pagos ...', 'medianetpay') . '",
                overlayCSS:
        {
            background: "#fff",
                opacity: 0.6
    },
    css: {
        padding:        20,
            textAlign:      "center",
            color:          "#555",
            border:         "3px solid #aaa",
            backgroundColor:"#fff",
            cursor:         "wait",
            lineHeight:"32px"
    }
    });
    jQuery("#submit_medianetpay_payment_form").click();});</script>
            </form>';
        }


        function process_payment($order_id)
        {
            $order = new WC_Order($order_id);

            $redirect_url = ($this->redirect_page_id == "" || $this->redirect_page_id == 0) ? get_site_url() . "/" : get_permalink($this->redirect_page_id);
            $redirect_url = $redirect_url . "?referencia=" . $order_id;

            $params = [
                'order' => $order_id,
                'usuario' => $this->merchant_id,
                'llavemd5' => $this->settings['salt'],
                'nombre' => $order->billing_first_name,
                'apellidos' => $order->billing_last_name,
                'total' => $order->order_total,
                'moneda' => $order->order_currency,
                'email' => $order->billing_email,
                'telefono' => $order->billing_phone,
                'referencia' => $order_id,
                'Direccion' => $order->billing_address_1,
                'descripcion' => 'Pago Orden de Compra Nro.' . $order_id,
                'gateway_url' => urlencode($this->liveurl),
                'urlredirect' => $redirect_url
            ];
            $url = plugin_dir_url(__FILE__) . "start.php?" . http_build_query($params);

            return array('result' => 'success', 'redirect' => $url);
        }

        function check_medianetpay_response()
        {
            global $woocommerce;
            if (isset($_REQUEST['txnid']) && isset($_REQUEST['mihpayid'])) {
                $order_id_time = $_REQUEST['txnid'];
                $order_id = explode('_', $_REQUEST['txnid']);
                $order_id = (int)$order_id[0];
                if ($order_id != '') {
                    try {
                        $order = new WC_Order($order_id);
                        $merchant_id = $_REQUEST['key'];
                        $amount = $_REQUEST['Amount'];
                        $hash = $_REQUEST['hash'];

                        $status = $_REQUEST['status'];
                        $productinfo = "Order $order_id";
                        echo $hash;
                        echo "{$this->salt}|$status|||||||||||{$order->billing_email}|{$order->billing_first_name}|$productinfo|{$order->order_total}|$order_id_time|{$this->merchant_id}";
                        $checkhash = hash('sha512', "{$this->salt}|$status|||||||||||{$order->billing_email}|{$order->billing_first_name}|$productinfo|{$order->order_total}|$order_id_time|{$this->merchant_id}");
                        $transauthorised = false;
                        if ($order->status !== 'completed') {
                            if ($hash == $checkhash) {

                                $status = strtolower($status);

                                if ($status == "success") {
                                    $transauthorised = true;
                                    $this->msg['message'] = "Thank you for shopping with us. Your account has been charged and your transaction is successful. We will be shipping your order to you soon.";
                                    $this->msg['class'] = 'woocommerce_message';
                                    if ($order->status == 'processing') {
                                    } else {
                                        $order->payment_complete();
                                        $order->add_order_note('MedianetPay payment successful<br/>Unnique Id from MedianetPay: ' . $_REQUEST['payid']);
                                        $order->add_order_note($this->msg['message']);
                                        $woocommerce->cart->empty_cart();
                                    }
                                } else if ($status == "pending") {
                                    $this->msg['message'] = "Thank you for shopping with us. Right now your payment staus is pending, We will keep you posted regarding the status of your order through e-mail";
                                    $this->msg['class'] = 'woocommerce_message woocommerce_message_info';
                                    $order->add_order_note('MedianetPay payment status is pending<br/>Unnique Id from MedianetPay: ' . $_REQUEST['payid']);
                                    $order->add_order_note($this->msg['message']);
                                    $order->update_status('on-hold');
                                    $woocommerce->cart->empty_cart();
                                } else {
                                    $this->msg['class'] = 'woocommerce_error';
                                    $this->msg['message'] = "Thank you for shopping with us. However, the transaction has been declined.";
                                    $order->add_order_note('Transaction Declined: ' . $_REQUEST['Error']);
                                }
                            } else {
                                $this->msg['class'] = 'error';
                                $this->msg['message'] = "Security Error. Illegal access detected";
                            }
                            if ($transauthorised == false) {
                                $order->update_status('failed');
                                $order->add_order_note('Failed');
                                $order->add_order_note($this->msg['message']);
                            }
                            add_action('the_content', array(&$this, 'showMessage'));
                        }
                    } catch (Exception $e) {

                        $msg = "Error";
                    }
                }
            }
        }

        function showMessage($content)
        {
            return '<div class="box ' . $this->msg['class'] . '-box">' . $this->msg['message'] . '</div>' . $content;
        }

        // get all pages
        function get_pages($title = false, $indent = true)
        {
            $wp_pages = get_pages('sort_column=menu_order');
            $page_list = array();
            if ($title) $page_list[] = $title;
            foreach ($wp_pages as $page) {
                $prefix = '';
                // show indented child pages?
                if ($indent) {
                    $has_parent = $page->post_parent;
                    while ($has_parent) {
                        $prefix .= ' - ';
                        $next_page = get_page($has_parent);
                        $has_parent = $next_page->post_parent;
                    }
                }
                // add to page list array array
                $page_list[$page->ID] = $prefix . $page->post_title;
            }
            return $page_list;
        }

        public static function showConfirmation()
        {
            global $woocommerce;
            $order = new WC_Order($_REQUEST['referencia']);

            wp_enqueue_script('fontawesome', 'https://kit.fontawesome.com/aa3a015ea3.js');
            wp_script_add_data('fontawesome', array('crossorigin'), array('anonymous'));

            require_once "confirmation.php";
        }

        function get_api_key()
        {
            return $this->settings['salt'];
        }

        function get_merchant_id()
        {
            return $this->settings['merchant_id'];
        }
    }

    /**
     * Add the Gateway to WooCommerce
     **/
    function woocommerce_add_medianetpay_gateway($methods)
    {
        $methods[] = 'WC_MedianetPay';
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'woocommerce_add_medianetpay_gateway');
}
